<?php

declare(strict_types=1);

namespace ProxyManager\Proxy\Exception;

use RuntimeException;

/**
 * Remote object exception
 *
 * @author Vincent Blanchon <blanchon.vincent@gmail.com>
 * @license MIT
 */
class RemoteObjectException extends RuntimeException
{
}
