<?php

declare(strict_types=1);

namespace ProxyManager\Proxy;

/**
 * Null object marker
 *
 * @author Vincent Blanchon <blanchon.vincent@gmail.com>
 * @license MIT
 */
interface NullObjectInterface extends ProxyInterface
{
}
