<?php

declare(strict_types=1);

namespace ProxyManager\Proxy;

/**
 * Smart reference object marker
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
interface SmartReferenceInterface extends ProxyInterface
{
}
