<?php

declare(strict_types=1);

namespace ProxyManager\Exception;

/**
 * Base exception class for the proxy manager
 *
 * @author Marco Pivetta <ocramius@gmail.com>
 * @license MIT
 */
interface ExceptionInterface
{
}
